/* Copyright 2008-2017 Research Foundation State University of New York   */

/* This file is part of QUB Online.                                       */

/* QUB Online is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License as published by   */
/* the Free Software Foundation, either version 3 of the License, or      */
/* (at your option) any later version.                                    */

/* QUB Online is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License,     */
/* named LICENSE.txt, in the QUB Online program directory.  If not, see  */
/* <http://www.gnu.org/licenses/>.                                        */

#ifndef CALLBACK_REPORTSTREAM_H
#define CALLBACK_REPORTSTREAM_H

// begin_html
// This header wraps a callbk_reportfun and its data pointer up in a std::ostream.
// <br><br><b>Up:</b> <a href="index.html">Index</a>
// end_html

#include <iostream>
#include <sstream>
#include <fstream>
#include "callbk_reportfun.h"
#include "qubfast.h"

class QUBFAST_API_CLASS callbk_reportbuf : public std::streambuf
{
 public:
 callbk_reportbuf() : lineout( new std::ostringstream )
    { setp(0,0), setg(0,0,0); }
  
  ~callbk_reportbuf() { delete lineout; }
  
  void setTarget( callbk_reportfun cb, void *_data )
  {
    callback = cb;
    data = _data;
  }
  
 protected:
  virtual int overflow( int ch )
  {
    if ( ch != EOF ) {
      if ( ch == '\n' ) {
	if ( callback )
	  callback( lineout->str().c_str(), data );
	else
	  std::cout << lineout->str().c_str() << std::endl;
	delete lineout;
	lineout = new std::ostringstream;
      }
      else {
	(*lineout) << (char) ch;
      }
    }
    return ch;
  }
  
 private:
  callbk_reportfun callback;
  void *data;
  std::ostringstream *lineout;
};

class QUBFAST_API_CLASS callbk_reportstream : public std::ostream
{
 public:
 callbk_reportstream( callbk_reportfun cb, void *data )
   : std::basic_ostream<char,std::char_traits<char> >( new callbk_reportbuf )
    {
      ((callbk_reportbuf *) rdbuf())->setTarget( cb, data );
    }
  virtual ~callbk_reportstream()
    {
      delete rdbuf();
    }
};

#endif
